from glob import glob
from subprocess import run
from sys import executable

from numpy import mean, std
from pandas import DataFrame, ExcelWriter


def file_list(path):
    file_list = []
    for fname in glob(path):
        file_list.append(fname)
    return file_list


def create_dataframes():
    return [DataFrame() for _ in range(7)]


def process_output(results):
    distances = []
    time = []
    for element in results:
        for line in element:
            if line.startswith(bytes("Total distance:", encoding="utf-8")):
                line_elements = line.split(sep=bytes(":", encoding="utf-8"))
                distances.append(float(line_elements[1]))
            if line.startswith(bytes("Execution time:", encoding="utf-8")):
                line_elements = line.split(sep=bytes(":", encoding="utf-8"))
                time.append(float(line_elements[1]))
    return distances, time


def populate_dataframes(df, output_list, dataset):
    df_distances, df_time = process_output(output_list)
    df_dict = {
        "dataset": dataset.removeprefix("data/"),
        "media distancia": mean(df_distances),
        "desviacion distancia": std(df_distances),
        "media tiempo": mean(df_time),
        "desviacion tiempo": std(df_time),
    }
    df = df.append(df_dict, ignore_index=True)
    return df


def script_execution(filenames, df_list, iterations=2):
    script = "src/main.py"
    parameters = [
        ["genetic", "uniform", "generational"],
        ["genetic", "position", "generational"],
        ["geneti,c" "uniform", "stationary"],
        ["genetic," "position", "stationary"],
        ["memetic", "all"],
        ["memetic", "random"],
        ["memetic", "best"],
    ]
    populated_list = []
    for dataset in filenames:
        print(f"Running on dataset {dataset}")
        for df, params in zip(df_list, parameters):
            print(f"Running {params} algorithm")
            output_list = []
            for _ in range(iterations):
                output_cmd = run(
                    [executable, script, dataset, *params], capture_output=True
                ).stdout.splitlines()
                output_list.append(output_cmd)
            populated_list.append(populate_dataframes(df, output_list, dataset))
    return populated_list


def export_results(df_list):
    dataframes = {
        "Generational uniform genetic": df_list[0],
        "Generational position genetic": df_list[1],
        "Stationary uniform genetic": df_list[2],
        "Stationary position genetic": df_list[3],
        "All genes memetic": df_list[4],
        "Random genes memetic": df_list[5],
        "Best genes memetic": df_list[6],
    }
    writer = ExcelWriter(path="docs/algorithm-results.xlsx", engine="xlsxwriter")
    for name, df in dataframes.items():
        df.to_excel(writer, sheet_name=name, index=False)
        worksheet = writer.sheets[name]
        for index, column in enumerate(df):
            series = df[column]
            max_length = max(series.astype(str).str.len().max(), len(str(series.name)))
            worksheet.set_column(index, index, width=max_length + 5)
    writer.save()


def main():
    datasets = file_list(path="data/*.txt")
    df_list = create_dataframes()
    populated_df_list = script_execution(datasets, df_list)
    export_results(populated_df_list)


if __name__ == "__main__":
    main()
